/* Exercise 7.12 */
/* This program shuffles the deck and then deal five cards */
/* and then determines the hand kind */

# include <stdio.h>
# include <stdlib.h>
# include <time.h>
# include <conio.h>
# include <string.h>


/* define a new record type */
typedef
   struct
   {
     char *cSuit;
     char *cFace;
     int cOrder;
   } tHand;


/************ prototypes *************/

/* this procedure shuffles the deck */
void shuffle(int [4][13]);

/* this function checks if the card is already dealed or not */
int checkOrder(int, int, tHand [5]);

/* this procedure deals a five-cards hand */
void deal(int [4][13], char *wSuit[], char *wFace[], tHand [5]);

/* This procedure finds # of faces in the hand */
/* I use tHand struct again but only cFace and cOrder members are used. */
/* Here, cOrder member holds the # of cards which has the same face */
void findKinds(tHand [5], tHand [5]);

/* this function determines if the hand contains one card pair */
int handContainsOnePair(tHand [5]);

/* this function determines if the hand contains two card pair */
int handContainsTwoPair(tHand [5]);

/* this function determines if the hand contains three same face card */
int handContainsThreeKind(tHand [5]);

/* this function determines if the hand contains four same face card */
int handContainsFourKind(tHand [5]);

/* this function determines if the hand is flush or not */
/* i.e., all the cards are same suit or not */
int handIsFlush(tHand [5]);

/* this function determines if the hand is straight or not */
/* i.e., all the cards are same suit and consecutive faces */
int handIsStraight(tHand [5], char *wFace[]);



main()
{
   char *suit[4] = {"Hearts", "Diamonds", "Clubs", "Spades"};
   char *face[13] = {"Ace", "Deuce", "Three", "Four", "Five", "Six", "Seven",
		     "Eight", "Nine", "Ten", "Jack", "Queen", "King"};
   int i, j, deck[4][13];
   tHand hand[5], cards[5];


   do
   {
      clrscr();

      for (i = 0 ; i <= 3 ; i++)
	 for (j = 0 ; j <= 12 ; j++)
	    deck[i][j] = 0;

      for (i = 0 ; i <= 4 ; i++)
      {
	 hand[i].cSuit = "";
	 hand[i].cFace = "";
	 hand[i].cOrder = 0;

	 cards[i].cSuit = "";
	 cards[i].cFace = "";
	 cards[i].cOrder = 0;
      }


      srand((unsigned) time(NULL));

      shuffle(deck);

      deal(deck, suit, face, hand);

      findKinds(hand, cards);


      printf("\n\n");

      for (i = 0 ; i <= 4 ; i++)
	 printf("%-10s  %-8s          (Card order in deck: %d)\n",
	    hand[i].cSuit, hand[i].cFace, hand[i].cOrder);


      printf("\n\n");

      printf("The hand %s a pair\n",
	 handContainsOnePair(cards) ? "contains" : "doesn't contain");
      printf("The hand %s two pair\n",
	 handContainsTwoPair(cards) ? "contains" : "doesn't contain");
      printf("The hand %s three kind\n",
	 handContainsThreeKind(cards) ? "contains" : "doesn't contain");
      printf("The hand %s four kind\n",
	 handContainsFourKind(cards) ? "contains" : "doesn't contain");

      printf("The hand %s flush\n",
	 handIsFlush(hand) ? "is" : "is not");
      printf("The hand %s straight\n",
	 handIsStraight(hand, face) ? "is" : "is not");

      printf("\n\nPress any key to continue, ESC to exit...");


   } while (getch() != 27);

   return 0;
}


/* this procedure shuffles the deck */
void shuffle(int wDeck[4][13])
{
   int card, row, column;

   for (card = 1 ; card <= 52 ; card++)
   {
      do
      {
	 row = rand() % 4;
	 column = rand() % 13;
      } while (wDeck[row][column] != 0);

      wDeck[row][column] = card;
   }

   return;
}


/* this function checks if the card is already dealed or not */
int checkOrder(int indx, int order, tHand oHand[5])
{
   int cnt;

   for (cnt = 0 ; cnt <= indx ; cnt++)
    if (oHand[cnt].cOrder == order)
       return 1;

   return 0;
}


/* this procedure deals a five-cards hand */
void deal(int wDeck[4][13], char *wSuit[], char *wFace[], tHand wHand[5])
{
   int cnt, card, row, column;


   for (cnt = 0 ; cnt <= 4 ; cnt++)
   {
      do
	card = (rand() % 52) + 1;
      while (checkOrder(cnt, card, wHand));

      wHand[cnt].cOrder = card;
   }

   for (cnt = 0 ; cnt <= 4 ; cnt++)
   {
      for (row = 0 ; row <= 3 ; row++)
	for (column = 0 ; column <= 12 ; column++)
	  if (wDeck[row][column] == wHand[cnt].cOrder)
	  {
	     wHand[cnt].cSuit = wSuit[row];
	     wHand[cnt].cFace = wFace[column];
	     break;
	  }
   }

   return;
}


/* This procedure finds # of faces in the hand */
/* I use tHand struct again but only cFace and cOrder members are used. */
/* Here, cOrder member holds the # of cards which has the same face */
void findKinds(tHand wHand[5], tHand wCards[5])
{
   int isFound, i, j;


   for (i = 0 ; i <= 4 ; i++)
   {
      isFound = 0;
      for (j = 0 ; j <= 4 ; j++)
      {
	 if (wHand[i].cFace == wCards[j].cFace)
	 {
	    wCards[j].cOrder++;
	    isFound = 1;
	 }

	 if (wCards[j].cFace[0] == '\0')
	    break;
      }

      if (! isFound)
      {
	 wCards[j].cFace =  wHand[i].cFace;
	 wCards[j].cOrder = 1;
      }
   }

   return;
}


/* this function determines if the hand contains one card pair */
int handContainsOnePair(tHand wCards[5])
{
   int i, onePairCount = 0;

   for (i = 0 ; i <= 4 ; i++)
      if (wCards[i].cOrder == 2)
	 onePairCount++;


   return (onePairCount == 1);
}


/* this function determines if the hand contains two card pair */
int handContainsTwoPair(tHand wCards[5])
{
   int i, twoPairCount = 0;

   for (i = 0 ; i <= 4 ; i++)
      if (wCards[i].cOrder == 2)
	 twoPairCount++;


   return (twoPairCount == 2);
}


/* this function determines if the hand contains three same face card */
int handContainsThreeKind(tHand wCards[5])
{
   int i;

   for (i = 0 ; i <= 4 ; i++)
      if (wCards[i].cOrder == 3)
	 return 1;

   return 0;
}


/* this function determines if the hand contains four same face card */
int handContainsFourKind(tHand wCards[5])
{
   int i;

   for (i = 0 ; i <= 4 ; i++)
      if (wCards[i].cOrder == 4)
	 return 1;

   return 0;
}


/* this function determines if the hand is flush or not */
/* i.e., all the cards are same suit or not */
int handIsFlush(tHand wHand[5])
{
   int i;
   char *suitName;

   suitName = wHand[0].cSuit;

   for (i = 1 ; i <= 4 ; i++)
      if (strcmp(wHand[i].cSuit, suitName) != 0)
	 return 0;

   return 1;
}


/* this function determines if the hand is straight or not */
/* i.e., all the cards are same suit and consecutive faces */
int handIsStraight(tHand wHand[5], char *wFace[])
{
   int cardIndex[5], i, j, Index, IsFound;

   if (handIsFlush(wHand))
   {
      for (i = 0 ; i <= 4 ; i++)
	 for (j = 0 ; j <= 12 ; j++)
	    if (strcmp(wHand[i].cFace, wFace[j]) == 0)
	       if (j == 0)
		  cardIndex[i] = 14;
	       else
		  cardIndex[i] = j + 1;

      Index = 15;
      for (i = 0 ; i <= 4 ; i++)
	 if (cardIndex[i] < Index)
	    Index = cardIndex[i];

      if (Index > 10)
	 return 0;

      for (i = 1 ; i <= 4 ; i++)
      {
	 Index++;
	 IsFound = 0;

	 for (j = 0 ; j <= 4 ; j++)
	    if (cardIndex[j] == Index)
	    {
	       IsFound = 1;
	       break;
	    }

	 if (! IsFound)
	    return 0;
      }

      return 1;
   }

   return 0;
}


